#|
 Copyright (c) 2008 Daniel Herring.
 Distributed under the Boost Software License, Version 1.0.
 (See accompanying file LICENSE_1_0.txt or copy at
  http://www.boost.org/LICENSE_1_0.txt)
|#

;(require :read-macros)

(defpackage :read-macros-examples
  (:use :cl))

(in-package :read-macros-examples)


;;;;
;;;; conditional reads
;;;;

(defun test-read-if ()
  (and
   (= #?(read-if t 1 2)
      1)
   (= #?(read-if nil 1 2)
      2)))

(defun test-read-when ()
  (and
   (equal (list #?(read-when t 1))
          '(1))
   (equal (list #?(read-when nil 1))
          nil)))

(defun test-read-unless ()
  (and
   (equal (list #?(read-unless t 1))
          nil)
   (equal (list #?(read-unless nil 1))
          '(1))))

(defun test-read-cond ()
  (and
   (equal (list #?(read-cond t 1 t 2))
          '(1))
   (equal (list #?(read-cond t 1 nil 2))
          '(1))
   (equal (list #?(read-cond nil 1 t 2))
          '(2))
   (equal (list #?(read-cond nil 1 nil 2))
          nil)))


;;;;
;;;; conditional reads based on *features*
;;;;

(defun test-eval-features ()
  (and
   ;; raw features
   (read-macros:eval-features (car *features*))
   (not (read-macros:eval-features (gensym)))
   ;; and
   (read-macros:eval-features '(:and))
   (read-macros:eval-features (list :and
                                    (first *features*)
                                    (second *features*)))
   (not (read-macros:eval-features (list :and
                                         (first *features*)
                                         (gensym))))
   ;; or
   (not (read-macros:eval-features '(:or)))
   (not (read-macros:eval-features (list :or
                                         (gensym))))
   (read-macros:eval-features (list :or
                                    (first *features*)
                                    (gensym)))
   ;; not
   (read-macros:eval-features '(:not (:or)))
   (not (read-macros:eval-features '(:not (:and))))))

(defun test-read-if-feature ()
  (and
   (= #?(read-if-feature (and) 1 2)
      1)
   (= #?(read-if-feature (or) 1 2)
      2)))

(defun test-read-when-feature ()
  (and
   (equal (list #?(read-when-feature (and) 1))
          '(1))
   (equal (list #?(read-when-feature (or) 1))
          nil)))

(defun test-read-unless-feature ()
  (and
   (equal (list #?(read-unless-feature (and) 1))
          nil)
   (equal (list #?(read-unless-feature (or) 1))
          '(1))))

(defun test-read-cond-features ()
  (and
   (equal (list #?(read-cond-features (and) 1 (and) 2))
          '(1))
   (equal (list #?(read-cond-features (and) 1 (or) 2))
          '(1))
   (equal (list #?(read-cond-features (or) 1 (and) 2))
          '(2))
   (equal (list #?(read-cond-features (or) 1 (or) 2))
          nil)))


;;;;
;;;; stateful reads similar to CPP's #if/#elif/#else/#endif
;;;;

(defun test-read-block ()
  (and
   (equal (list 0
                #?(read-block-if t) 1
                #?(read-block-else-if t) 2
                #?(read-block-else) 3
                #?(read-block-end)
                4)
          '(0 1 4))
   (equal (list 0
                #?(read-block-if t) 1
                #?(read-block-else-if nil) 2
                #?(read-block-else) 3
                #?(read-block-end)
                4)
          '(0 1 4))
   (equal (list 0
                #?(read-block-if nil) 1
                #?(read-block-else-if t) 2
                #?(read-block-else) 3
                #?(read-block-end)
                4)
          '(0 2 4))
   (equal (list 0
                #?(read-block-if nil) 1
                #?(read-block-else-if nil) 2
                #?(read-block-else) 3
                #?(read-block-end)
                4)
          '(0 3 4))))

;;;;
;;;; lexical read manipulation
;;;;

(defun test-read-let ()
  (and
   (= #?(read-let ((*read-base* 16))
                  (+ 10 20))
      (* 3 16))
   (eq #?(read-let ((*read-suppress* t))
                   (+ 10 20))
       nil)))


;;;;
;;;; here documents
;;;;

(defun test-read-here-string ()
  (and
   (string=
    #?(read-here-string _EOF)
 this is 'a "test" of #?(read-here-string)
 (this is only a test)
_EOF
    " this is 'a \"test\" of #?(read-here-string)
 (this is only a test)")

   (string=
    #?(read-here-string)
 one more time

    " one more time")))
